
local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Options")

----------------------------
-- Locals

local ACD = LibStub("AceConfigDialog-3.0")
local LoD = RW:GetPlugin("LoD")
local AceGUI = LibStub("AceGUI-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("RW-2.0")
local LSM = LibStub("LibSharedMedia-3.0")

local iconCoords = {
	STAR		= {0, 		0.25,	0, 		0.25},
	CIRCLE 		= {0.25,	0.5,	0,		0.25},
	DIAMOND		= {0.5, 	0.75,	0,		0.25},
	TRIANGLE 	= {0.75,	1,		0,		0.25},
	MOON 		= {0,		0.25,	0.25,	0.5},
	SQUARE 		= {0.25,	0.5,	0.25,	0.5},
	CROSS 		= {0.5,		0.75,	0.25,	0.5},
	SKULL 		= {0.75,	1,		0.25,	0.5},
}

local OptionsFrame
local DetailsFrame
local bosses
local bossMods
local orderedBossMods
local modIcons

local defaults = {
	profile = {
		VersionSort = "name",
	}
}

----------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Options", defaults)
	
	self.BossOptions = {BossMods = {}}
	self.PluginOptions = {Plugins = {}}
		
	bosses = {}
	bossMods = {}
	orderedBossMods = {}
	modIcons = {}
	
	self:RegisterMessage("RW_BossModFound")
	self:RegisterMessage("RW_BossRegistered")
	
	self.options = {
		type = "group",
		name = "Raid Watch 2",
		handler = self,
		args = {
			General = {
				type = "group",
				name = "General",
				order = 1,
				args = {
					Config = {
						type = "execute",
						name = "Config",
						func = "ToggleConfigMode",
					},
					MMIcon = {
						type = "toggle",
						name = "Hide minimap button",
						desc = "Turn the minimap icon on and off",
						get = function() return RW.db.profile.minimap.hide end,
						set = function(info, value)
							RW.db.profile.minimap.hide = value
							local icon = LibStub("LibDBIcon-1.0")
							if not icon then return end
							
							if RW.db.profile.minimap.hide then
								icon:Hide("RW2")
							else
								icon:Show("RW2")
							end
						end,
					},
				}
			},
			Plugins = {	
				type = "group",
				name = "Plugins",
				order = 3,
				plugins = self.PluginOptions,
				args = {}
			},
			Profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(RW.db),
		}
	}	
end

function plugin:PluginEnable()
	self.NextConfigMode = true
	LibStub("AceConfig-3.0"):RegisterOptionsTable("RW", self.options)
	
	self:RegisterMessage("RW_BossRegistered")
	self:RegisterMessage("RW_BossModFound")
	self:RegisterMessage("RW_PlayerLeftParty", "PartyUpdate")
	self:RegisterMessage("RW_PayerJoinedParty", "PartyUpdate")
	self:RegisterMessage("RW_JoinedParty", "PartyUpdate")
	self:RegisterMessage("RW_LeftGroup", "PartyUpdate")	
	
	self:SendMessage("RW_RegisterCommListener", self, "IncVersionInfo")
	
	for modName, modID in pairs(RW.modStorage) do
		bossMods[modName] = modID
		if not RW:InTable(orderedBossMods, modName) then
			tinsert(orderedBossMods, modName)
		end
	end
	
	for name, icon in pairs(RW.iconStorage) do
		modIcons[name] = icon
	end
	
	for i, boss in pairs(RW.bossStorage) do
		bosses[boss.modPack] = bosses[boss.modPack] or {}
		if not RW:InTable(bosses[boss.modPack], boss) then
			tinsert(bosses[boss.modPack], boss)
		end
	end
end

function plugin:PluginDisable()
end

function plugin:ToggleConfigMode()
	RW:SendMessage("RW_ConfigMode", self.NextConfigMode) 
	self.NextConfigMode = not self.NextConfigMode
end

----------------------
-- Message handlers --
----------------------
function plugin:RW_BossRegistered(message, boss)
	bosses[boss.modPack] = bosses[boss.modPack] or {}
	tinsert(bosses[boss.modPack], boss)
end

function plugin:RW_BossModFound(message, modName, modID)
	bossMods[modName] = modID or true
	if not RW:InTable(orderedBossMods, modName) then
		tinsert(orderedBossMods, modName)
	end
end
----------------------

do
	local treeSelectPath = {}
	local treeStatusTable = {}
	local treeLastSelected	
	
	local bossList = {}
	local bossListLastSelected = {}
	local bossHeaders = {}
	local addedOptions = {}
	
	------------------------------
	-- Option Details
	
	local function showDetailTab(tabs, event, group)
		tabs:ReleaseChildren()
		local baseObj = tabs:GetUserData("baseObj")
		
		local scroll = AceGUI:Create("ScrollFrame")
		scroll:SetLayout("Flow")
		scroll:SetFullWidth(true)
		scroll:SetFullHeight(true)	

		local g = AceGUI:Create("CategoryFrame")
		g:SetFullWidth(true)
		g:SetTitle("")		
		
		local added = false
		for i, obj in pairs(baseObj.mod.AlertObjects[group]) do
			if obj.base == baseObj then
				added = true
				g:AddChild(obj:GetOption())
			end
		end
		
		scroll:AddChild(g)
		tabs:AddChild(scroll)
	end
	
	-- Returns the tabs for the details about the selected option
	local tabs = {}
	local function getOptionTabs(baseObj)
		wipe(tabs)
		for i, b in pairs(RW.BITS) do
			if baseObj.mod.AlertObjects[i] then
				local c = false
				for _, obj in pairs(baseObj.mod.AlertObjects[i]) do
					if obj.base == baseObj then
						c = true
						break
					end
				end
				if c then 
					tinsert(tabs, {value = i, text = L[i]})
				end
			end
		end
		return tabs
	end
	
	local function showHideDetails(baseObj)
		if DetailsFrame then 
			DetailsFrame:Release()
			DetailsFrame = nil
		end
			
		local optText = type(baseObj.id) == "number" and RW.SpellName[baseObj.id] or baseObj.id
			
		DetailsFrame = AceGUI:Create("RWFrame")
		DetailsFrame:SetLayout("Fill")
		DetailsFrame.frame:ClearAllPoints()
		DetailsFrame.frame:SetPoint("LEFT", OptionsFrame.frame, "RIGHT", 5, 0)
		DetailsFrame:SetWidth(500)
		DetailsFrame:SetHeight(450)
		DetailsFrame:SetTitle(optText)
		DetailsFrame:SetCallback("OnClose", function(widget)
			widget:ReleaseChildren()
			widget:Release()
			DetailsFrame = nil
		end)
		
		local t = getOptionTabs(baseObj)
		local tab = AceGUI:Create("TabGroup")
		tab:SetTabs(t)
		tab:SetUserData("baseObj", baseObj)
		tab:SetCallback("OnGroupSelected", showDetailTab)
		tab:SelectTab(t[1] and t[1].value or nil)
		tab:SetCallback("OnTabEnter", function(widget, event, key, tab)
				GameTooltip:SetOwner(widget.frame)
				GameTooltip:AddLine(L[key])
				GameTooltip:AddLine(L[key.."_DESC"])
				GameTooltip:Show()
			end)
		tab:SetCallback("OnTabLeave", function() GameTooltip:Hide() end)
		
		DetailsFrame:AddChild(tab)
		
		DetailsFrame:Show()
	end
	
	------------------------------
	-- Boss Detials
	
	local function getMasterValue(baseObj)
		if baseObj.type == "bool" then
			return baseObj.mod.DB[baseObj.var]
		else
			return baseObj.mod.DB[baseObj.id].enabled
		end
	end
	
	local function toggleMaster(check, event, value)
		local baseObj = check:GetUserData("baseObj")
		
		if baseObj.type == "bool" then
			baseObj.mod.DB[baseObj.var] = not baseObj.mod.DB[baseObj.var]
		else
			baseObj.mod.DB[baseObj.id].enabled = not baseObj.mod.DB[baseObj.id].enabled
		end
		
		if DetailsFrame then
			showHideDetails(baseObj)
		end
	end
	
	local function GetOptionGroup(opt, boss)
		local baseObj = boss:GetBaseAlertObject(opt)
		--[===[@debug@
		if not baseObj then
			--error(("%s: No option connected to %q"):format(boss:GetName(), opt))
		end
		--@end-debug@]===]		
		
		local optText = type(opt) == "number" and RW.SpellName[opt] or opt
	
		local group = AceGUI:Create("OptionFrame")
		group:SetLayout("Flow")
		group:SetRelativeWidth(0.5)
		--group:SetAutoAdjustHeight(false)
		--group:SetHeight(80)		
		group:SetTitle(optText)
		
		if baseObj then
			local check = AceGUI:Create("CheckBox")
			check:SetLabel("")
			check:SetUserData("baseObj", baseObj)
			check:SetWidth(20)
			check:SetValue(getMasterValue(baseObj))
			check:SetCallback("OnValueChanged", toggleMaster)
			group:AddChild(check)
		
			local icon = AceGUI:Create("Icon")
			icon:SetText("")
			icon:SetImage(baseObj.icon)
			icon:SetImageSize(35, 35)
			icon:SetHeight(40)
			icon:SetWidth(65)
			icon:SetUserData("baseObj", baseObj)
			
			if baseObj.type ~= "bool" then
				icon:SetCallback("OnClick", function(self, event, button)
					local baseObj = self:GetUserData("baseObj")
					if button == "LeftButton" then
						if IsShiftKeyDown() and ChatFrameEditBox:IsShown() then
							if baseObj.spell then
								ChatFrameEditBox:Insert("\124cff71d5ff\124Hspell:"..baseObj.spell.."\124h["..RW.SpellName[baseObj.spell].."]\124h\124r")
							end
						else
							showHideDetails(baseObj)
						end
					end
				end)
			end
			
			if baseObj.spell then
				icon:SetCallback("OnEnter", function(self)
					GameTooltip:SetOwner(self.frame)
					GameTooltip:SetHyperlink("spell:"..baseObj.spell)
					GameTooltip:AddLine(" ")
					GameTooltip:AddLine("Click to show detailed settings")
					GameTooltip:AddLine("[Shift + Left click to add link in chat]")
					GameTooltip:Show()
				end)
				icon:SetCallback("OnLeave", function()
					GameTooltip:Hide()
				end)
			end
			group:AddChild(icon)
			
			local iconObj = boss:GetIconObject(opt)
			if iconObj then
				local iconGroup = AceGUI:Create("SimpleGroup")
				iconGroup:SetLayout("Flow")
				iconGroup:SetRelativeWidth(0.5)
				group:AddChild(iconGroup)
				
				for i in pairs(iconObj.icons) do
					local icon = AceGUI:Create("Icon")
					icon:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[i]))
					icon:SetImageSize(18, 18)
					icon:SetHeight(20)
					icon:SetWidth(25)
					iconGroup:AddChild(icon)
				end
			end
		end
		
		return group
	end
	
	local function selectBoss(drop, event, group)
		drop:ReleaseChildren()
		
		boss = bosses[drop:GetUserData("bossMod")][group]
		if not boss then error(("Bossmod %q is not loaded"):format(drop:GetUserData("bossMod"))) end
		drop:SetUserData("boss", boss)
		
		bossListLastSelected[drop:GetUserData("bossMod")] = group
		
		local scroll = AceGUI:Create("ScrollFrame")
		scroll:SetFullWidth(true)
		scroll:SetFullHeight(true)
		scroll:SetLayout("Flow")

		local miscGroup = AceGUI:Create("CategoryFrame")
		miscGroup:SetLayout("Flow")
		miscGroup:SetTitle("Misc")
		miscGroup:SetRelativeWidth(0.5)
		scroll:AddChild(miscGroup)
		
		local enableCheck = AceGUI:Create("CheckBox")
		enableCheck:SetLabel("Enabled")
		enableCheck:SetHeight(35)
		enableCheck:SetValue(boss.db.profile.enabled)
		miscGroup:AddChild(enableCheck)
		
		if boss.usedIcons then
			local iconGroup = AceGUI:Create("CategoryFrame")
			iconGroup:SetLayout("Flow")
			iconGroup:SetTitle("Icons used")
			iconGroup:SetRelativeWidth(0.5)
			scroll:AddChild(iconGroup)
			
			for i in pairs(boss.usedIcons) do
				local icon = AceGUI:Create("Icon")
				icon:SetImage("Interface\\TargetingFrame\\UI-RaidTargetingIcons.blp", unpack(iconCoords[i]))
				icon:SetImageSize(20, 20)
				icon:SetRelativeWidth(0.124)
				icon:SetHeight(35)
				iconGroup:AddChild(icon)
			end
		end
		
		if boss.Options then
			-- Build options titles
			wipe(bossHeaders)
			if boss.Options.Categories then
				for i, cat in pairs(boss.Options.Categories) do
					local group = AceGUI:Create("CategoryFrame")
					group:SetLayout("Flow")
					group:SetTitle(cat.text)
					group:SetFullWidth(true)
					scroll:AddChild(group)
					bossHeaders[cat.text] = group
					
					for j, opt in ipairs(cat) do
						local optGroup = GetOptionGroup(opt, boss)
						if optGroup then
							group:AddChild(optGroup)
						end
					end
				end
			end			
		end		
		
		drop:AddChild(scroll)
	end
	
	------------------------------
	-- Boss Selection
	
	local function getStatistics(tab, event, group)
		tab:ReleaseChildren()
		tab:PauseLayout()
	
		local modName = tab:GetUserData("modName")
			
		-- constants
		local tTextSize = 14
		local collWidth = 0.16
	
		local header = AceGUI:Create("Label")
		header:SetRelativeWidth(0.3)
		header:SetFont(LSM:Fetch("font", "accid"), tTextSize)
		header:SetColor(0.78, 0.9, 0.16)
		header:SetText("Boss")
		tab:AddChild(header)
		
		header = AceGUI:Create("Label")
		header:SetRelativeWidth(collWidth)
		header:SetFont(LSM:Fetch("font", "accid"), tTextSize)
		header:SetColor(0.78, 0.9, 0.16)
		header:SetText("Kills")
		tab:AddChild(header)
		
		header = AceGUI:Create("Label")
		header:SetRelativeWidth(collWidth)
		header:SetFont(LSM:Fetch("font", "accid"), tTextSize)
		header:SetColor(0.78, 0.9, 0.16)
		header:SetText("Wipes")
		tab:AddChild(header)
		
		header = AceGUI:Create("Label")
		header:SetRelativeWidth(collWidth)
		header:SetFont(LSM:Fetch("font", "accid"), tTextSize)
		header:SetColor(0.78, 0.9, 0.16)
		header:SetText("Total")
		tab:AddChild(header)
		
		header = AceGUI:Create("Label")
		header:SetRelativeWidth(collWidth)
		header:SetFont(LSM:Fetch("font", "accid"), tTextSize)
		header:SetColor(0.78, 0.9, 0.16)
		header:SetText("Fastest")
		tab:AddChild(header)
		
		local heading = AceGUI:Create("Heading")
		heading:SetFullWidth(true)
		tab:AddChild(heading)
		
		local color1 = {1, 0.6, 0.16}
		local color2 = {1, 1, 1}
		
		for i, boss in pairs(bosses[modName]) do
			if not boss.TrashMod then
		
				local stats = boss.DB.Stats[group]

				local color
				if i % 2 == 0 then
					color = color1
				else
					color = color2
				end
				
				local line = AceGUI:Create("Label")
				line:SetRelativeWidth(0.30)
				line:SetFont(LSM:Fetch("font", "accid"), tTextSize)
				line:SetColor(unpack(color))
				if boss.L then
					line:SetText(boss.L.name)
				else
					line:SetText(boss:GetName())
				end				
				tab:AddChild(line)
				
				line = AceGUI:Create("Label")
				line:SetRelativeWidth(collWidth)
				line:SetFont(LSM:Fetch("font", "accid"), tTextSize)
				line:SetColor(unpack(color))
				line:SetText((stats["heroic"].kills).." / "..(stats["normal"].kills))
				tab:AddChild(line)
				
				line = AceGUI:Create("Label")
				line:SetRelativeWidth(collWidth)
				line:SetFont(LSM:Fetch("font", "accid"), tTextSize)
				line:SetColor(unpack(color))
				line:SetText((stats["heroic"].wipes).." / "..(stats["normal"].wipes))
				tab:AddChild(line)
				
				local hTotal = stats["heroic"].kills + stats["heroic"].wipes
				local nTotal = stats["normal"].kills + stats["normal"].wipes
				
				line = AceGUI:Create("Label")
				line:SetRelativeWidth(collWidth)
				line:SetFont(LSM:Fetch("font", "accid"), tTextSize)
				line:SetColor(unpack(color))
				line:SetText(hTotal.." / "..nTotal)
				tab:AddChild(line)
				
				local hFastest = stats["heroic"].best or 0
				local nFastest = stats["normal"].best or 0
				
				line = AceGUI:Create("Label")
				line:SetRelativeWidth(collWidth)
				line:SetFont(LSM:Fetch("font", "accid"), tTextSize)
				line:SetColor(unpack(color))
				line:SetText((hFastest > 0 and RW:TimeToStringShort(hFastest) or "~").." / "..(nFastest > 0 and RW:TimeToStringShort(nFastest) or "~"))
				tab:AddChild(line)		
			end
		end
		tab:ResumeLayout()
		tab:DoLayout()
	end
	
	local function buildBossList(modName, subGroup)
		local firstNr = nil
		wipe(bossList)
		for	i, boss in pairs(bosses[modName]) do
			if boss.subGroup == subGroup then
				if not firstNr then firstNr = i end
				bossList[i] = boss.L and boss.L.name or boss:GetName()
			end
		end
		return bossList, firstNr
	end
	
	local function selectTreeOption(tree, event, group)
		tree:SetUserData("Using", nil)
		tree:ReleaseChildren()
		treeLastSelected = group
		
		-- Only make AceGUI configuration if its a boss mod
		-- Else open host the AceConfig inside this window
		if select(1, string.split("\001", group)) == "Profiles" then
			local scroll = AceGUI:Create("ScrollFrame")
			scroll:SetLayout("flow")
			scroll.width = "fill"
			scroll.height = "fill"
			
			local filler = AceGUI:Create("SimpleGroup")
			filler:SetLayout("Flow")
			filler.width = "fill"
			
			ACD:Open("RW", filler, "Profiles")
			scroll:AddChild(filler)
			tree:AddChild(scroll)
			
		elseif select(1, string.split("\001", group)) == "Plugins" or select(1, string.split("\001", group)) == "General" then
			wipe(treeSelectPath)
			for i = 1, select("#", string.split("\001", group)) do
				pathPart = select(i, string.split("\001", group))
				if not pathPart:find("category") then 
					tinsert(treeSelectPath, pathPart)
				end
			end
		
			if #treeSelectPath == 1 and treeSelectPath[1] == "Plugins" then return end
			
			local subGroups = false
			if treeSelectPath[1] == "Plugins" then
				for _, child in pairs(plugin.PluginOptions.Plugins[treeSelectPath[2]].args) do
					if child.type == "group" and not child.guiInline then
						subGroups = true
						break
					end
				end
			end
			
			local addFrame
			
			if subGroups then				
				addFrame = AceGUI:Create("SimpleGroup")
				addFrame:SetLayout("Fill")
				addFrame.width = "fill"
				addFrame.height = "fill"	
				
				ACD:Open("RW", addFrame, unpack(treeSelectPath))
				tree:AddChild(addFrame)
			else
				local scroll = AceGUI:Create("ScrollFrame")
				scroll:SetLayout("flow")
				scroll.width = "fill"
				scroll.height = "fill"
				
				addFrame = AceGUI:Create("SimpleGroup")
				addFrame:SetLayout("Fill")
				addFrame.width = "fill"
				addFrame.height = "fill"
							
				ACD:Open("RW", addFrame, unpack(treeSelectPath))
				scroll:AddChild(addFrame)
				tree:AddChild(scroll)
			end			
			
		-- Boss statistics page
		elseif group == "BossMods" then
			local titel = AceGUI:Create("Label")
			titel:SetFont(LSM:Fetch("font", "accid"), 25, "OUTLINE")
			titel:SetText("Statistics")
			titel:SetHeight(40)
		
			scroll = AceGUI:Create("ScrollFrame")
			scroll:SetLayout("Flow")
			
			tree:AddChild(scroll)
			
			scroll:AddChild(titel)
		
			local statTabs = {
				{ value = 25, text = "25 man" },
				{ value = 10, text = "10 man" },
			}
			
			local partyTab = {
				{value = 25, text = "Party"}
			}
			
			for i, modName in pairs(orderedBossMods) do				
				if bosses[modName] then			
					local modType = bosses[modName][1].ModType or "Raid"
					
					local sgroup = AceGUI:Create("SimpleGroup")
					sgroup:SetLayout("Flow")
					sgroup:SetFullWidth(true)
					
					local Icon = AceGUI:Create("Icon")
					Icon:SetImage(modIcons[modName])
					Icon:SetImageSize(40, 40)
					Icon:SetWidth(45)
					Icon:SetHeight(40)
					
					local label = AceGUI:Create("Label")
					label:SetRelativeWidth(0.8)
					--label:SetImage(modIcons[modName])
					label:SetFont(LSM:Fetch("font", "accid"), 20, "OUTLINE")
					label:SetColor(0.78, 0.6, 0.16)
					label:SetText(modName)
					
					local inGroup = AceGUI:Create("TabGroup")
					inGroup:SetTabs(modType == "Raid" and statTabs or partyTab)				
					inGroup:SetFullWidth(true)
					inGroup:SetLayout("Flow")	
					inGroup:SetUserData("modName", modName)
					inGroup:SetCallback("OnGroupSelected", getStatistics)
					inGroup:SelectTab((modType == "Party" or modType == "Raid") and 25 or 10)
					
					sgroup:AddChild(Icon)
					sgroup:AddChild(label)
					sgroup:AddChild(inGroup)
					
					scroll:AddChild(sgroup)
				end
			end
			
		elseif group == "Versions" then
		
			plugin:GetVersionDisplay(tree)
		
		else
			-- Get the full path minus "BossMods"
			nodes = { select(2, string.split("\001", group)) }
			if #nodes < 1 then return end
			
			local bossMod, subGroup
			-- This mod has sub groups
			if #nodes > 1 then
				bossMod = select(4, nodes[#nodes - 1]:find("type:([%w%s-]*);?([%w%s-]*)"))
				subGroup = select(4, nodes[#nodes]:find("type:([%w%s-]*);?([%w%s-]*)"))
			-- This mod have no subgroups
			else
				bossMod = select(4, nodes[#nodes]:find("type:([%w%s-]*);?([%w%s-]*)"))
			end
			-- This mod is loaded
			if bosses[bossMod] then
				nameList, firstNr = buildBossList(bossMod, subGroup)
				if firstNr then
					dropdown = AceGUI:Create("DropdownGroup")
					dropdown:SetLayout("Flow")
					dropdown:SetUserData("bossMod", bossMod)
					dropdown:SetFullWidth(true)
					dropdown:SetFullHeight(true)
					dropdown:SetGroupList(nameList)
					dropdown:SetCallback("OnGroupSelected", selectBoss)
					if bossListLastSelected[bossMod] and nameList[bossListLastSelected[bossMod]] then
						dropdown:SetGroup(bossListLastSelected[bossMod])
					else
						dropdown:SetGroup(firstNr)
					end
					tree:AddChild(dropdown)
				else
					local label = AceGUI:Create("Label")
					label:SetFullWidth(true)
					label:SetFont(LSM:Fetch("font", "accid"), 25)
					label:SetText("This mod has sub sections, please klick the plus sign to the right of the mod name and select a section there.")
					tree:AddChild(label)					
				end
			-- Not loaded, (add load button)
			else
				local button = AceGUI:Create("Button")
				button:SetText("Load")
				button:SetUserData("modID", bossMods[bossMod])
				button:SetUserData("modName", bossMod)
				button:SetUserData("tree", tree)
				button:SetUserData("path", group)
				button:SetCallback("OnClick", function(widget, event)
					if not IsAddOnLoaded(widget:GetUserData("modID")) then
						LoD:LoadMod(widget:GetUserData("modID"), widget:GetUserData("modName"))
						widget:GetUserData("tree"):SetTree(plugin:BuildOptionsTree())
						selectTreeOption(widget:GetUserData("tree"), "nil", widget:GetUserData("path"))
					end
				end)
				local sgroup = AceGUI:Create("SimpleGroup")
				sgroup:AddChild(button)
				tree:AddChild(sgroup)
			end
		end
	end

	
	-----------------------------
	-- Main window
	
	function plugin:Toggle()
		if OptionsFrame then
			OptionsFrame:Release()
			OptionsFrame = nil
		else	
			OptionsFrame = AceGUI:Create("RWFrame")
			OptionsFrame:SetTitle("Raid Watch 2")
			OptionsFrame:SetLayout("Fill")
			OptionsFrame:SetWidth(810)
			OptionsFrame:SetHeight(600)
			OptionsFrame:SetCallback("OnClose", function(widget)
				widget:ReleaseChildren()
				widget:Release()
				if DetailsFrame then
					DetailsFrame:Release()
					DetailsFrame = nil
				end
				OptionsFrame = nil
			end)
			OptionsFrame.frame:ClearAllPoints()
			OptionsFrame.frame:SetPoint("LEFT", UIParent, "LEFT", 50, 0)
			
			local tree = AceGUI:Create("TreeGroup")
			tree:SetLayout("Fill")
			tree:SetFullHeight(true)
			tree:SetTree(self:BuildOptionsTree())
			tree:SetCallback("OnGroupSelected", selectTreeOption)		
			tree:SetSelected(treeLastSelected or "General")	
			tree:SetStatusTable(treeStatusTable)
			OptionsFrame:AddChild(tree)
			
			OptionsFrame:Show()
		end
	end
end

do
	local catOrders = {}
	local TreeGroups = {}
	function plugin:BuildOptionsTree()
		wipe(TreeGroups)
		TreeGroups = {
			{
				value = "General",
				text = "General"
			},
			{
				value = "BossMods",
				text = "Boss Mods",
				children = {
				}
			},
			{
				value = "Plugins",
				text = "Plugins",
				children = {
				}
			},
			{
				value = "Profiles",
				text = "Profiles",
				children = {
				}
			},
			{
				value = "Versions",
				text = "Versions",
			},
		}
		
		TreeGroups[3].children = {
			{value = "category_1", text = "Alert Plugins", icon = [[Interface\Icons\INV_Sigil_UlduarAll]], children = {}},
			{value = "category_2", text = "Misc Plugins", icon = [[Interface\Icons\Achievement_Boss_CThun]], children = {}},
		}
		catOrders["Alert Plugins"] = 1
		catOrders["Misc Plugins"] = 2
		
		for i, module in RW:IteratePlugins() do
			if module.GetOptions then
				local pluginOptions = module:GetOptions()
				self.PluginOptions.Plugins[module:GetName()] = pluginOptions
				-- Module has a category
				if module.category then
					-- First time category seen this round
					if not catOrders[module.category] then
						TreeGroups[3].children[#TreeGroups[3].children + 1] = {value = "category_"..module.category, text = module.category, children = {}}
						catOrders[module.category] = #TreeGroups[3].children
					-- Category seen before but not added yet
					elseif not TreeGroups[3].children[catOrders[module.category]] then
						TreeGroups[3].children[catOrders[module.category]] = {value = "category_"..module.category, text = module.category, children = {}}
					end
					-- Add module to the category
					tinsert(TreeGroups[3].children[catOrders[module.category]].children, {value = module:GetName(), text = pluginOptions.name, icon = pluginOptions.icon})			
				else
					tinsert(TreeGroups[3].children, {value = module:GetName(), text = pluginOptions.name})
				end
			end
		end
		
		for i, cat in pairs(TreeGroups[3].children) do
			if cat.children then
				table.sort(cat.children, function(a, b) return a.text < b.text end)
			end
		end
		
		local function checkList(lst, value)
			for i, t in pairs(lst) do
				if t.text == value then return false end
			end
			return true
		end
		
		for i, modName in pairs(orderedBossMods) do
			local modID = bossMods[modName]
			leafe = {value = "type:leafe;"..modName, text = modName, icon = modIcons[modName]}
			if bosses[modName] then
				for i, boss in pairs(bosses[modName]) do
					if boss.icon then		
						leafe.icon = boss.icon
						modIcons[boss.modPack] = boss.icon
					end
					if boss.subGroup then
						if not leafe.children or checkList(leafe.children, boss.subGroup) then
							leafe.value = "type:subGroup;"..modName
							leafe.children = leafe.children or {}
							tinsert(leafe.children, {value = "type:leafe;"..boss.subGroup, text = boss.subGroup})
						end
					end
				end
			end		
			leafe.icon = modIcons[modName]
			tinsert(TreeGroups[2].children, leafe)
		end
				
		return TreeGroups
	end
end

--------------------------
-- Option helpers

plugin.OptionHelpers = {}
function plugin.OptionHelpers.OnOptionEnter(self, event)
	local obj = self:GetUserData("obj")
	GameTooltip:SetOwner(self.frame)
	GameTooltip:AddLine(L[obj.mainType.."_"..obj.type])
	GameTooltip:AddLine(L[obj.mainType.."_"..obj.type.."_DESC"]:format(obj.base.id))
	GameTooltip:Show()
end

function plugin.OptionHelpers.OnOptionLeave(self, event)
	GameTooltip:Hide()
end

function plugin.OptionHelpers.OnColorSelected(self, event, r, g, b, a)
	local obj = self:GetUserData("obj")
	local db = obj.base.mod.DB
	db[obj.base.id][obj.mainType][obj.optText or obj.type].color = {r, g, b, a}
end

function plugin.OptionHelpers.OnValueChanged(self, event, value)
	local obj = self:GetUserData("obj")
	local db = obj.base.mod.DB
	db[obj.base.id][obj.mainType][obj.optText or obj.type].enabled = value
end

function plugin.OptionHelpers.OnKeyValueChanged(self, event, value)
	local obj, key = self:GetUserData("obj"), self:GetUserData("key")
	local db = obj.base.mod.DB
	db[obj.base.id][obj.mainType][obj.optText or obj.type][key] = value
end