local plugin = RW:GetPlugin("Options")
if not plugin then return end

local AceGUI = LibStub("AceGUI-3.0")
local LSM = LibStub("LibSharedMedia-3.0")
local party
local partyNames = {}
local guildNames = {}
local selectedSortMethod
local displayTree
local updateSchedule

local sortmethods = {
	["name"] = "By Name",
	["version"] = "By Version",
}

local function sortByName(a, b)
	return a.name < b.name
end

local function sortByVersion(a, b)
	return (a.version or "999") < (b.version or "999")
end

local function selectSortMethod(widget, event, method)
	if method == "name" then
		selectedSortMethod = sortByName
	elseif method == "version" then
		selectedSortMethod = sortByVersion
	end
	
	plugin.db.profile.VersionSort = method
end

selectedSortMethod = sortByName

local function updateParty()
	party = party or RW:GetPlugin("Party")
	wipe(partyNames)
	if party:GetPartyType() ~= "none" then
		
		for name, data in party:PartyIterator() do
			tinsert(partyNames, data)
		end
		table.sort(partyNames, selectedSortMethod)
	end
end

local function addNames(listType, list, group)
	local sgroup = AceGUI:Create("InlineGroup")
	sgroup:SetTitle(listType)
	sgroup:SetLayout("Flow")
	sgroup:SetFullWidth(true)
	
	local col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
	col:SetText("Name")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
	col:SetText("Version")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
	col:SetText("Revisions")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	col = AceGUI:Create("Label")
	col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
	col:SetText("Type")
	col:SetRelativeWidth(0.25)
	sgroup:AddChild(col)
	
	local sep = AceGUI:Create("Label")
	sep:SetText(" ")
	sep:SetFullWidth(true)
	sgroup:AddChild(sep)
	
	for i, data in pairs(list) do
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
		col:SetText(data.name)
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
		col:SetText(data.version or "---")
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
		col:SetText(data.revision or "---")
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
		
		col = AceGUI:Create("Label")
		col:SetFont(LSM:Fetch("font", "accid"), 13, "OUTLINE")
		col:SetText(data.releaseType or "---")
		col:SetRelativeWidth(0.25)
		sgroup:AddChild(col)
	end
	
	group:AddChild(sgroup)
end

function plugin:GetVersionDisplay(tree)
	displayTree = tree
	tree:SetUserData("Using", "VersionDisplay")
	tree:ReleaseChildren()
		
	local group = AceGUI:Create("ScrollFrame")
		
	local titel = AceGUI:Create("Label")
	titel:SetFont(LSM:Fetch("font", "accid"), 25, "OUTLINE")
	titel:SetText("Versions")
	titel:SetFullWidth(true)	
	group:AddChild(titel)
	
	local sortSelect = AceGUI:Create("Dropdown")
	sortSelect:SetCallback("OnValueChanged", selectSortMethod)
	sortSelect:SetList(sortmethods)
	sortSelect:SetValue(self.db.profile.VersionSort)
	sortSelect:SetLabel("Sorting")
	group:AddChild(sortSelect)
	
	updateParty()
	if #partyNames > 0 then
		addNames("Party", partyNames, group)		
	end
	
	if #guildNames > 0 then
		addNames("Guild", guildNames, group)	
	end

	tree:AddChild(group)
end

function plugin:UpdateVersionDisplay()
	if displayTree ~= nil and (displayTree:GetUserData("Using") == "VersionDisplay") then
		displayTree:ReleaseChildren()
		plugin:GetVersionDisplay(displayTree)
	end	
	updateSchedule = nil
end

function plugin:PartyUpdate(message)
	plugin:UpdateVersionDisplay()
end

function plugin:IncVersionInfo(sender, dist, version, revision, releaseType)
	if dist == "GUILD" then
		tinsert(guildNames, {name = sender, version = version, revision = revision, releaseType = releaseType})
		table.sort(guildNames, selectedSortMethod)
	end
	if not updateSchedule then
		updateSchedule = self:ScheduleTimer("UpdateVersionDisplay", 2)
	end
end